@@ -45,7 +45,6 @@
 #ifdef XP_WIN
 #include "mozilla/a11y/Compatibility.h"
 #include "mozilla/dom/ContentChild.h"
-#include "HTMLWin32ObjectAccessible.h"
 #include "mozilla/StaticPtr.h"
 #endif
 
@@ -58,7 +57,6 @@
 #include "nsINamed.h"
 #include "nsIObserverService.h"
 #include "nsLayoutUtils.h"
-#include "nsPluginFrame.h"
 #include "SVGGeometryFrame.h"
 #include "nsTreeBodyFrame.h"
 #include "nsTreeColumns.h"
@@ -84,10 +82,6 @@
 #include "XULTreeGridAccessibleWrap.h"
 #endif
 
-#if defined(XP_WIN) || defined(MOZ_ACCESSIBILITY_ATK)
-#include "nsNPAPIPluginInstance.h"
-#endif
-
 using namespace mozilla;
 using namespace mozilla::a11y;
 using namespace mozilla::dom;
@@ -105,20 +99,22 @@ MustBeAccessible(nsIContent* aContent, DocAccessible* aDocument)
   if (aContent->GetPrimaryFrame()->IsFocusable())
     return true;
 
-  uint32_t attrCount = aContent->GetAttrCount();
-  for (uint32_t attrIdx = 0; attrIdx < attrCount; attrIdx++) {
-    const nsAttrName* attr = aContent->GetAttrNameAt(attrIdx);
-    if (attr->NamespaceEquals(kNameSpaceID_None)) {
-      nsIAtom* attrAtom = attr->Atom();
-      nsDependentAtomString attrStr(attrAtom);
-      if (!StringBeginsWith(attrStr, NS_LITERAL_STRING("aria-")))
-        continue; // not ARIA
+  if (aContent->IsElement()) {
+    uint32_t attrCount = aContent->AsElement()->GetAttrCount();
+    for (uint32_t attrIdx = 0; attrIdx < attrCount; attrIdx++) {
+      const nsAttrName* attr = aContent->AsElement()->GetAttrNameAt(attrIdx);
+      if (attr->NamespaceEquals(kNameSpaceID_None)) {
+        nsIAtom* attrAtom = attr->Atom();
+        nsDependentAtomString attrStr(attrAtom);
+        if (!StringBeginsWith(attrStr, NS_LITERAL_STRING("aria-")))
+          continue; // not ARIA
 
-      // A global state or a property and in case of token defined.
-      uint8_t attrFlags = aria::AttrCharacteristicsFor(attrAtom);
-      if ((attrFlags & ATTR_GLOBAL) && (!(attrFlags & ATTR_VALTOKEN) ||
-           nsAccUtils::HasDefinedARIAToken(aContent, attrAtom))) {
-        return true;
+        // A global state or a property and in case of token defined.
+        uint8_t attrFlags = aria::AttrCharacteristicsFor(attrAtom);
+        if ((attrFlags & ATTR_GLOBAL) && (!(attrFlags & ATTR_VALTOKEN) ||
+             nsAccUtils::HasDefinedARIAToken(aContent, attrAtom))) {
+          return true;
+        }
       }
     }
   }
@@ -206,6 +202,14 @@ static Accessible* New_HTMLInput(nsIContent* aContent, Accessible* aContext)
                             nsGkAtoms::radio, eIgnoreCase)) {
     return new HTMLRadioButtonAccessible(aContent, aContext->Document());
   }
+  if (aContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
+                            nsGkAtoms::time, eIgnoreCase)) {
+    return new EnumRoleAccessible<roles::GROUPING>(aContent, aContext->Document());
+  }
+  if (aContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
+                            nsGkAtoms::date, eIgnoreCase)) {
+    return new EnumRoleAccessible<roles::DATE_EDITOR>(aContent, aContext->Document());
+  }
   return nullptr;
 }
 
@@ -310,24 +314,12 @@ nsAccessibilityService::ListenersChanged(nsIArray* aEventChanges)
     if (!node || !node->IsHTMLElement()) {
       continue;
     }
-    nsCOMPtr<nsIArray> listenerNames;
-    change->GetChangedListenerNames(getter_AddRefs(listenerNames));
 
     uint32_t changeCount;
-    rv = listenerNames->GetLength(&changeCount);
+    change->GetCountOfEventListenerChangesAffectingAccessibility(&changeCount);
     NS_ENSURE_SUCCESS(rv, rv);
 
     for (uint32_t i = 0 ; i < changeCount ; i++) {
-      nsCOMPtr<nsIAtom> listenerName = do_QueryElementAt(listenerNames, i);
-
-      // We are only interested in event listener changes which may
-      // make an element accessible or inaccessible.
-      if (listenerName != nsGkAtoms::onclick &&
-          listenerName != nsGkAtoms::onmousedown &&
-          listenerName != nsGkAtoms::onmouseup) {
-        continue;
-      }
-
       nsIDocument* ownerDoc = node->OwnerDoc();
       DocAccessible* document = GetExistingDocAccessible(ownerDoc);
 
@@ -408,120 +400,6 @@ nsAccessibilityService::GetRootDocumentAccessible(nsIPresShell* aPresShell,
   return nullptr;
 }
 
-#ifdef XP_WIN
-static StaticAutoPtr<nsTArray<nsCOMPtr<nsIContent> > > sPendingPlugins;
-static StaticAutoPtr<nsTArray<nsCOMPtr<nsITimer> > > sPluginTimers;
-
-class PluginTimerCallBack final : public nsITimerCallback
-                                , public nsINamed
-{
-  ~PluginTimerCallBack() {}
-
-public:
-  explicit PluginTimerCallBack(nsIContent* aContent) : mContent(aContent) {}
-
-  NS_DECL_ISUPPORTS
-
-  NS_IMETHOD Notify(nsITimer* aTimer) final override
-  {
-    if (!mContent->IsInUncomposedDoc())
-      return NS_OK;
-
-    nsIPresShell* ps = mContent->OwnerDoc()->GetShell();
-    if (ps) {
-      DocAccessible* doc = ps->GetDocAccessible();
-      if (doc) {
-        // Make sure that if we created an accessible for the plugin that wasn't
-        // a plugin accessible we remove it before creating the right accessible.
-        doc->RecreateAccessible(mContent);
-        sPluginTimers->RemoveElement(aTimer);
-        return NS_OK;
-      }
-    }
-
-    // We couldn't get a doc accessible so presumably the document went away.
-    // In this case don't leak our ref to the content or timer.
-    sPendingPlugins->RemoveElement(mContent);
-    sPluginTimers->RemoveElement(aTimer);
-    return NS_OK;
-  }
-
-  NS_IMETHOD GetName(nsACString& aName) final override
-  {
-    aName.AssignLiteral("PluginTimerCallBack");
-    return NS_OK;
-  }
-
-private:
-  nsCOMPtr<nsIContent> mContent;
-};
-
-NS_IMPL_ISUPPORTS(PluginTimerCallBack, nsITimerCallback, nsINamed)
-#endif
-
-already_AddRefed<Accessible>
-nsAccessibilityService::CreatePluginAccessible(nsPluginFrame* aFrame,
-                                               nsIContent* aContent,
-                                               Accessible* aContext)
-{
-  // nsPluginFrame means a plugin, so we need to use the accessibility support
-  // of the plugin.
-  if (aFrame->GetRect().IsEmpty())
-    return nullptr;
-
-#if defined(XP_WIN) || defined(MOZ_ACCESSIBILITY_ATK)
-  RefPtr<nsNPAPIPluginInstance> pluginInstance;
-  if (NS_SUCCEEDED(aFrame->GetPluginInstance(getter_AddRefs(pluginInstance))) &&
-      pluginInstance) {
-#ifdef XP_WIN
-    if (!sPendingPlugins->Contains(aContent) &&
-        (Preferences::GetBool("accessibility.delay_plugins") ||
-         Compatibility::IsJAWS() || Compatibility::IsWE())) {
-      nsCOMPtr<nsITimer> timer = do_CreateInstance(NS_TIMER_CONTRACTID);
-      RefPtr<PluginTimerCallBack> cb = new PluginTimerCallBack(aContent);
-      timer->InitWithCallback(cb, Preferences::GetUint("accessibility.delay_plugin_time"),
-                              nsITimer::TYPE_ONE_SHOT);
-      sPluginTimers->AppendElement(timer);
-      sPendingPlugins->AppendElement(aContent);
-      return nullptr;
-    }
-
-    // We need to remove aContent from the pending plugins here to avoid
-    // reentrancy.  When the timer fires it calls
-    // DocAccessible::ContentInserted() which does the work async.
-    sPendingPlugins->RemoveElement(aContent);
-
-    // Note: pluginPort will be null if windowless.
-    HWND pluginPort = nullptr;
-    aFrame->GetPluginPort(&pluginPort);
-
-    RefPtr<Accessible> accessible =
-      new HTMLWin32ObjectOwnerAccessible(aContent, aContext->Document(),
-                                         pluginPort);
-    return accessible.forget();
-
-#elif MOZ_ACCESSIBILITY_ATK
-    if (!AtkSocketAccessible::gCanEmbed)
-      return nullptr;
-
-    // Note this calls into the plugin, so crazy things may happen and aFrame
-    // may go away.
-    nsCString plugId;
-    nsresult rv = pluginInstance->GetValueFromPlugin(
-      NPPVpluginNativeAccessibleAtkPlugId, &plugId);
-    if (NS_SUCCEEDED(rv) && !plugId.IsEmpty()) {
-      RefPtr<AtkSocketAccessible> socketAccessible =
-        new AtkSocketAccessible(aContent, aContext->Document(), plugId);
-
-      return socketAccessible.forget();
-    }
-#endif
-  }
-#endif
-
-  return nullptr;
-}
-
 void
 nsAccessibilityService::DeckPanelSwitched(nsIPresShell* aPresShell,
                                           nsIContent* aDeckNode,
@@ -672,7 +550,7 @@ nsAccessibilityService::UpdateListBullet(nsIPresShell* aPresShell,
 void
 nsAccessibilityService::UpdateImageMap(nsImageFrame* aImageFrame)
 {
-  nsIPresShell* presShell = aImageFrame->PresContext()->PresShell();
+  nsIPresShell* presShell = aImageFrame->PresShell();
   DocAccessible* document = GetDocAccessible(presShell);
   if (document) {
     Accessible* accessible =
@@ -1304,9 +1182,11 @@ nsAccessibilityService::Init()
       // obtain a MSAA content process id.
       contentChild->SendGetA11yContentId();
     }
-#endif // defined(XP_WIN)
 
+    gApplicationAccessible = new ApplicationAccessibleWrap();
+#else
     gApplicationAccessible = new ApplicationAccessible();
+#endif // defined(XP_WIN)
   }
 
   NS_ADDREF(gApplicationAccessible); // will release in Shutdown()
@@ -1316,11 +1196,6 @@ nsAccessibilityService::Init()
     AnnotateCrashReport(NS_LITERAL_CSTRING("Accessibility"),
                         NS_LITERAL_CSTRING("Active"));
 
-#ifdef XP_WIN
-  sPendingPlugins = new nsTArray<nsCOMPtr<nsIContent> >;
-  sPluginTimers = new nsTArray<nsCOMPtr<nsITimer> >;
-#endif
-
   // Now its safe to start platform accessibility.
   if (XRE_IsParentProcess())
     PlatformInit();
@@ -1357,16 +1232,6 @@ nsAccessibilityService::Shutdown()
 
   SelectionManager::Shutdown();
 
-#ifdef XP_WIN
-  sPendingPlugins = nullptr;
-
-  uint32_t timerCount = sPluginTimers->Length();
-  for (uint32_t i = 0; i < timerCount; i++)
-    sPluginTimers->ElementAt(i)->Cancel();
-
-  sPluginTimers = nullptr;
-#endif
-
   if (XRE_IsParentProcess())
     PlatformShutdown();
 
@@ -1682,11 +1547,6 @@ nsAccessibilityService::CreateAccessibleByFrameType(nsIFrame* aFrame,
     case eOuterDocType:
       newAcc = new OuterDocAccessible(aContent, document);
       break;
-    case ePluginType: {
-      nsPluginFrame* pluginFrame = do_QueryFrame(aFrame);
-      newAcc = CreatePluginAccessible(pluginFrame, aContent, aContext);
-      break;
-    }
     case eTextLeafType:
       newAcc = new TextLeafAccessibleWrap(aContent, document);
       break;
