@@ -79,6 +79,11 @@ DocAccessibleParent::RecvShowEvent(const ShowEventData& aData,
 
   MOZ_ASSERT(CheckDocTree());
 
+  // Just update, no events.
+  if (aData.EventSuppressed()) {
+    return IPC_OK();
+  }
+
   ProxyAccessible* target = parent->ChildAt(newChildIdx);
   ProxyShowHideEvent(target, parent, true, aFromUser);
 
@@ -697,38 +702,6 @@ DocAccessibleParent::SetEmulatedWindowHandle(HWND aWindowHandle)
   mEmulatedWindowHandle = aWindowHandle;
 }
 
-mozilla::ipc::IPCResult
-DocAccessibleParent::RecvGetWindowedPluginIAccessible(
-      const WindowsHandle& aHwnd, IAccessibleHolder* aPluginCOMProxy)
-{
-#if defined(MOZ_CONTENT_SANDBOX)
-  // We don't actually want the accessible object for aHwnd, but rather the
-  // one that belongs to its child (see HTMLWin32ObjectAccessible).
-  HWND childWnd = ::GetWindow(reinterpret_cast<HWND>(aHwnd), GW_CHILD);
-  if (!childWnd) {
-    // We're seeing this in the wild - the plugin is windowed but we no longer
-    // have a window.
-    return IPC_OK();
-  }
-
-  IAccessible* rawAccPlugin = nullptr;
-  HRESULT hr = ::AccessibleObjectFromWindow(childWnd, OBJID_WINDOW,
-                                            IID_IAccessible,
-                                            (void**)&rawAccPlugin);
-  if (FAILED(hr)) {
-    // This might happen if the plugin doesn't handle WM_GETOBJECT properly.
-    // We should not consider that a failure.
-    return IPC_OK();
-  }
-
-  aPluginCOMProxy->Set(IAccessibleHolder::COMPtrType(rawAccPlugin));
-
-  return IPC_OK();
-#else
-  return IPC_FAIL(this, "Message unsupported in this build configuration");
-#endif
-}
-
 mozilla::ipc::IPCResult
 DocAccessibleParent::RecvFocusEvent(const uint64_t& aID,
                                     const LayoutDeviceIntRect& aCaretRect)
