@@ -1,9 +1,9 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
+from __future__ import absolute_import
 from __future__ import print_function
 import os
-import sys
 from mozbuild.util import FileAvoidWrite
 
 header_template = '''#pragma GCC system_header
