@@ -366,7 +366,7 @@ OutputParser.prototype = {
 
         case "ident":
           if (options.expectCubicBezier &&
-              BEZIER_KEYWORDS.indexOf(token.text) >= 0) {
+              BEZIER_KEYWORDS.includes(token.text)) {
             this._appendCubicBezier(token.text, options);
           } else if (Services.prefs.getBoolPref(CSS_GRID_ENABLED_PREF) &&
                      this._isDisplayGrid(text, token, options)) {
