@@ -51,6 +51,8 @@ namespace dom {
 
 using namespace workers;
 
+static LazyLogModule gEventSourceLog("EventSource");
+
 #define SPACE_CHAR           (char16_t)0x0020
 #define CR_CHAR              (char16_t)0x000D
 #define LF_CHAR              (char16_t)0x000A
@@ -953,8 +955,7 @@ EventSourceImpl::SetupHttpChannel()
 {
   AssertIsOnMainThread();
   MOZ_ASSERT(!IsShutDown());
-  DebugOnly<nsresult> rv =
-    mHttpChannel->SetRequestMethod(NS_LITERAL_CSTRING("GET"));
+  nsresult rv = mHttpChannel->SetRequestMethod(NS_LITERAL_CSTRING("GET"));
   MOZ_ASSERT(NS_SUCCEEDED(rv));
 
   /* set the http request headers */
@@ -965,11 +966,19 @@ EventSourceImpl::SetupHttpChannel()
 
   // LOAD_BYPASS_CACHE already adds the Cache-Control: no-cache header
 
-  if (!mLastEventID.IsEmpty()) {
-    rv = mHttpChannel->SetRequestHeader(NS_LITERAL_CSTRING("Last-Event-ID"),
-      NS_ConvertUTF16toUTF8(mLastEventID), false);
-    MOZ_ASSERT(NS_SUCCEEDED(rv));
+  if (mLastEventID.IsEmpty()) {
+    return;
   }
+  NS_ConvertUTF16toUTF8 eventId(mLastEventID);
+  rv = mHttpChannel->SetRequestHeader(NS_LITERAL_CSTRING("Last-Event-ID"),
+                                      eventId, false);
+#ifdef DEBUG
+  if (NS_FAILED(rv)) {
+    MOZ_LOG(gEventSourceLog, LogLevel::Warning,
+            ("SetupHttpChannel. rv=%x (%s)", uint32_t(rv), eventId.get()));
+  }
+#endif
+  Unused << rv;
 }
 
 nsresult
@@ -1205,7 +1214,7 @@ EventSourceImpl::SetReconnectionTimeout()
 
   // the timer will be used whenever the requests are going finished.
   if (!mTimer) {
-    mTimer = do_CreateInstance("@mozilla.org/timer;1");
+    mTimer = NS_NewTimer();
     NS_ENSURE_STATE(mTimer);
   }
 
@@ -1728,7 +1737,8 @@ EventSourceImpl::ParseCharacter(char16_t aChr)
         NS_ENSURE_SUCCESS(rv, rv);
 
         mStatus = PARSE_STATE_BEGIN_OF_LINE;
-      } else {
+      } else if (aChr != 0) {
+        // Avoid appending the null char to the field value.
         mLastFieldValue += aChr;
       }
 
@@ -1747,7 +1757,8 @@ EventSourceImpl::ParseCharacter(char16_t aChr)
         mStatus = PARSE_STATE_BEGIN_OF_LINE;
       } else if (aChr == COLON_CHAR) {
         mStatus = PARSE_STATE_COMMENT;
-      } else {
+      } else if (aChr != 0) {
+        // Avoid appending the null char to the field name.
         mLastFieldName += aChr;
         mStatus = PARSE_STATE_FIELD_NAME;
       }
@@ -1775,7 +1786,8 @@ class EventSourceWorkerHolder final : public WorkerHolder
 {
 public:
   explicit EventSourceWorkerHolder(EventSourceImpl* aEventSourceImpl)
-    : mEventSourceImpl(aEventSourceImpl)
+    : WorkerHolder("EventSourceWorkerHolder")
+    , mEventSourceImpl(aEventSourceImpl)
   {
   }
 
