@@ -111,6 +111,7 @@ const char* mozilla::dom::ContentPrefs::gInitPrefs[] = {
   "javascript.options.native_regexp",
   "javascript.options.parallel_parsing",
   "javascript.options.shared_memory",
+  "javascript.options.spectre.index_masking",
   "javascript.options.streams",
   "javascript.options.strict",
   "javascript.options.strict.debug",
@@ -123,21 +124,21 @@ const char* mozilla::dom::ContentPrefs::gInitPrefs[] = {
   "javascript.options.werror",
   "javascript.use_us_english_locale",
   "jsloader.shareGlobal",
+#ifdef MOZ_STYLO
+  "layout.css.servo.chrome.enabled",
+  "layout.css.stylo-blocklist.blocked_domains",
+  "layout.css.stylo-blocklist.enabled",
+#endif
   "layout.idle_period.required_quiescent_frames",
   "layout.idle_period.time_limit",
   "layout.interruptible-reflow.enabled",
   "mathml.disabled",
-  "media.apple.forcevda",
   "media.clearkey.persistent-license.enabled",
   "media.cubeb.backend",
   "media.cubeb.sandbox",
   "media.cubeb_latency_msg_frames",
   "media.cubeb_latency_playback_ms",
   "media.decoder-doctor.wmf-disabled-is-failure",
-  "media.decoder.fuzzing.dont-delay-inputexhausted",
-  "media.decoder.fuzzing.enabled",
-  "media.decoder.fuzzing.video-output-minimum-interval-ms",
-  "media.decoder.limit",
   "media.decoder.recycle.enabled",
   "media.dormant-on-pause-timeout-ms",
   "media.eme.audio.blank",
@@ -148,18 +149,13 @@ const char* mozilla::dom::ContentPrefs::gInitPrefs[] = {
   "media.ffvpx.low-latency.enabled",
   "media.flac.enabled",
   "media.forcestereo.enabled",
-  "media.gmp.async-shutdown-timeout",
-  "media.gmp.decoder.aac",
   "media.gmp.decoder.enabled",
-  "media.gmp.decoder.h264",
   "media.gmp.insecure.allow",
   "media.gpu-process-decoder",
   "media.libavcodec.allow-obsolete",
-  "media.num-decode-threads",
   "media.ogg.enabled",
   "media.ogg.flac.enabled",
   "media.resampling.enabled",
-  "media.resampling.rate",
   "media.ruin-av-sync.enabled",
   "media.rust.test_mode",
   "media.suspend-bkgnd-video.delay-ms",
@@ -174,7 +170,6 @@ const char* mozilla::dom::ContentPrefs::gInitPrefs[] = {
   "media.webspeech.test.fake_fsm_events",
   "media.webspeech.test.fake_recognition_service",
   "media.wmf.allow-unsupported-resolutions",
-  "media.wmf.decoder.thread-count",
   "media.wmf.enabled",
   "media.wmf.skip-blacklist",
   "media.wmf.vp9.enabled",
@@ -224,7 +219,6 @@ const char* mozilla::dom::ContentPrefs::gInitPrefs[] = {
   "security.sandbox.logging.enabled",
   "security.sandbox.mac.track.violations",
   "security.sandbox.windows.log.stackTraceDepth",
-  "signed.applets.codebase_principal_support",
   "svg.disabled",
   "svg.display-lists.hit-testing.enabled",
   "svg.display-lists.painting.enabled",
