@@ -225,34 +225,12 @@ public:
                                     bool* aIsCommitted,
                                     nsString* aCommittedString) override;
 
-  virtual mozilla::ipc::IPCResult
-  RecvStartPluginIME(const WidgetKeyboardEvent& aKeyboardEvent,
-                     const int32_t& aPanelX,
-                     const int32_t& aPanelY,
-                     nsString* aCommitted) override;
-
-  virtual mozilla::ipc::IPCResult RecvSetPluginFocused(const bool& aFocused) override;
-
-  virtual mozilla::ipc::IPCResult RecvSetCandidateWindowForPlugin(
-    const widget::CandidateWindowPosition& aPosition) override;
-  virtual mozilla::ipc::IPCResult
-  RecvEnableIMEForPlugin(const bool& aEnable) override;
-
-  virtual mozilla::ipc::IPCResult
-  RecvDefaultProcOfPluginEvent(const WidgetPluginEvent& aEvent) override;
-
-  virtual mozilla::ipc::IPCResult RecvGetInputContext(int32_t* aIMEEnabled,
-                                                      int32_t* aIMEOpen) override;
-
-  virtual mozilla::ipc::IPCResult RecvSetInputContext(const int32_t& aIMEEnabled,
-                                                      const int32_t& aIMEOpen,
-                                                      const nsString& aType,
-                                                      const nsString& aInputmode,
-                                                      const nsString& aActionHint,
-                                                      const bool& aInPrivateBrowsing,
-                                                      const int32_t& aCause,
-                                                      const int32_t& aFocusChange) override;
+  virtual mozilla::ipc::IPCResult RecvGetInputContext(
+    widget::IMEState* aIMEState) override;
 
+  virtual mozilla::ipc::IPCResult RecvSetInputContext(
+    const widget::InputContext& aContext,
+    const widget::InputContextAction& aAction) override;
 
   virtual mozilla::ipc::IPCResult RecvRequestFocus(const bool& aCanRaise) override;
 
@@ -346,6 +324,8 @@ public:
 
   DimensionInfo GetDimensionInfo();
 
+  nsresult UpdatePosition();
+
   void SizeModeChanged(const nsSizeMode& aSizeMode);
 
   void UIResolutionChanged();
@@ -441,8 +421,6 @@ public:
 
   void SendRealTouchEvent(WidgetTouchEvent& aEvent);
 
-  void SendPluginEvent(WidgetPluginEvent& aEvent);
-
   /**
    * Different from above Send*Event(), these methods return true if the
    * event has been posted to the remote process or failed to do that but
@@ -537,14 +515,6 @@ public:
 
   LayoutDevicePoint AdjustTapToChildWidget(const LayoutDevicePoint& aPoint);
 
-  /**
-   * Native widget remoting protocol for use with windowed plugins with e10s.
-   */
-  virtual PPluginWidgetParent* AllocPPluginWidgetParent() override;
-
-  virtual bool
-  DeallocPPluginWidgetParent(PPluginWidgetParent* aActor) override;
-
   virtual PPaymentRequestParent*
   AllocPPaymentRequestParent() override;
 
@@ -650,8 +620,6 @@ private:
   RefPtr<nsIContentParent> mManager;
   void TryCacheDPIAndScale();
 
-  nsresult UpdatePosition();
-
   bool AsyncPanZoomEnabled() const;
 
   // Cached value indicating the docshell active state of the remote browser.
