@@ -145,6 +145,7 @@ bool
 nsMathMLElement::ParseAttribute(int32_t aNamespaceID,
                                 nsIAtom* aAttribute,
                                 const nsAString& aValue,
+                                nsIPrincipal* aMaybeScriptedPrincipal,
                                 nsAttrValue& aResult)
 {
   MOZ_ASSERT(IsMathMLElement());
@@ -177,7 +178,9 @@ nsMathMLElement::ParseAttribute(int32_t aNamespaceID,
   }
 
   return nsMathMLElementBase::ParseAttribute(aNamespaceID, aAttribute,
-                                             aValue, aResult);
+                                             aValue,
+                                             aMaybeScriptedPrincipal,
+                                             aResult);
 }
 
 static Element::MappedAttributeEntry sMtableStyles[] = {
@@ -1085,7 +1088,9 @@ nsMathMLElement::GetHrefURI() const
 nsresult
 nsMathMLElement::AfterSetAttr(int32_t aNameSpaceID, nsIAtom* aName,
                               const nsAttrValue* aValue,
-                              const nsAttrValue* aOldValue, bool aNotify)
+                              const nsAttrValue* aOldValue,
+                              nsIPrincipal* aSubjectPrincipal,
+                              bool aNotify)
 {
   // It is important that this be done after the attribute is set/unset.
   // We will need the updated attribute value because notifying the document
@@ -1103,7 +1108,7 @@ nsMathMLElement::AfterSetAttr(int32_t aNameSpaceID, nsIAtom* aName,
   }
 
   return nsMathMLElementBase::AfterSetAttr(aNameSpaceID, aName, aValue,
-                                           aOldValue, aNotify);
+                                           aOldValue, aSubjectPrincipal, aNotify);
 }
 
 JSObject*
