@@ -51,7 +51,7 @@ AudioSink::AudioSink(AbstractThread* aThread,
   bool resampling = MediaPrefs::AudioSinkResampling();
 
   if (resampling) {
-    mOutputRate = MediaPrefs::AudioSinkResampleRate();
+    mOutputRate = 48000;
   } else if (mInfo.mRate == 44100 || mInfo.mRate == 48000) {
     // The original rate is of good quality and we want to minimize unecessary
     // resampling. The common scenario being that the sampling rate is one or
@@ -424,28 +424,21 @@ AudioSink::NotifyAudioNeeded()
       missingFrames = std::min<int64_t>(INT32_MAX, missingFrames.value());
       mFramesParsed += missingFrames.value();
 
-      // We need to calculate how many frames are missing at the output rate.
-      missingFrames =
-        SaferMultDiv(missingFrames.value(), mOutputRate, data->mRate);
-      if (!missingFrames.isValid()) {
-        NS_WARNING("Int overflow in AudioSink");
-        mErrored = true;
-        return;
-      }
-
-      // We need to insert silence, first use drained frames if any.
-      missingFrames -= DrainConverter(missingFrames.value());
-      // Insert silence if still needed.
-      if (missingFrames.value()) {
-        AlignedAudioBuffer silenceData(missingFrames.value() * mOutputChannels);
-        if (!silenceData) {
-          NS_WARNING("OOM in AudioSink");
-          mErrored = true;
-          return;
-        }
-        RefPtr<AudioData> silence = CreateAudioFromBuffer(Move(silenceData), data);
-        PushProcessedAudio(silence);
+      RefPtr<AudioData> silenceData;
+      AlignedAudioBuffer silenceBuffer(missingFrames.value() * data->mChannels);
+       if (!silenceBuffer) {
+         NS_WARNING("OOM in AudioSink");
+         mErrored = true;
+         return;
+       }
+      if (mConverter->InputConfig() != mConverter->OutputConfig()) {
+        AlignedAudioBuffer convertedData =
+          mConverter->Process(AudioSampleBuffer(Move(silenceBuffer))).Forget();
+        silenceData = CreateAudioFromBuffer(Move(convertedData), data);
+      } else {
+        silenceData = CreateAudioFromBuffer(Move(silenceBuffer), data);
       }
+      PushProcessedAudio(silenceData);
     }
 
     mLastEndTime = data->GetEndTime();
@@ -486,7 +479,7 @@ AudioSink::PushProcessedAudio(AudioData* aData)
 
 already_AddRefed<AudioData>
 AudioSink::CreateAudioFromBuffer(AlignedAudioBuffer&& aBuffer,
-                                            AudioData* aReference)
+                                 AudioData* aReference)
 {
   uint32_t frames = aBuffer.Length() / mOutputChannels;
   if (!frames) {
@@ -542,5 +535,22 @@ AudioSink::DrainConverter(uint32_t aMaxFrames)
   return data->mFrames;
 }
 
+nsCString
+AudioSink::GetDebugInfo()
+{
+  MOZ_ASSERT(mOwnerThread->IsCurrentThreadIn());
+  return nsPrintfCString("AudioSink: StartTime=%" PRId64
+                         " LastGoodPosition=%" PRId64
+                         " Playing=%d  OutputRate=%u Written=%" PRId64
+                         " Errored=%d PlaybackComplete=%d",
+                         mStartTime.ToMicroseconds(),
+                         mLastGoodPosition.ToMicroseconds(),
+                         mPlaying,
+                         mOutputRate,
+                         mWritten,
+                         bool(mErrored),
+                         bool(mPlaybackComplete));
+}
+
 } // namespace media
 } // namespace mozilla
