@@ -363,7 +363,7 @@ AppleVTDecoder::OutputFrame(CVPixelBufferRef aImage,
     size_t width = CVPixelBufferGetWidth(aImage);
     size_t height = CVPixelBufferGetHeight(aImage);
     DebugOnly<size_t> planes = CVPixelBufferGetPlaneCount(aImage);
-    MOZ_ASSERT(planes == 2, "Likely not NV12 format and it must be.");
+    MOZ_ASSERT(planes == 3, "Likely not YUV420 format and it must be.");
 
     VideoData::YCbCrBuffer buffer;
 
@@ -393,15 +393,15 @@ AppleVTDecoder::OutputFrame(CVPixelBufferRef aImage,
     buffer.mPlanes[1].mWidth = (width+1) / 2;
     buffer.mPlanes[1].mHeight = (height+1) / 2;
     buffer.mPlanes[1].mOffset = 0;
-    buffer.mPlanes[1].mSkip = 1;
+    buffer.mPlanes[1].mSkip = 0;
     // Cr plane.
     buffer.mPlanes[2].mData =
-      static_cast<uint8_t*>(CVPixelBufferGetBaseAddressOfPlane(aImage, 1));
-    buffer.mPlanes[2].mStride = CVPixelBufferGetBytesPerRowOfPlane(aImage, 1);
+      static_cast<uint8_t*>(CVPixelBufferGetBaseAddressOfPlane(aImage, 2));
+    buffer.mPlanes[2].mStride = CVPixelBufferGetBytesPerRowOfPlane(aImage, 2);
     buffer.mPlanes[2].mWidth = (width+1) / 2;
     buffer.mPlanes[2].mHeight = (height+1) / 2;
-    buffer.mPlanes[2].mOffset = 1;
-    buffer.mPlanes[2].mSkip = 1;
+    buffer.mPlanes[2].mOffset = 0;
+    buffer.mPlanes[2].mSkip = 0;
 
     gfx::IntRect visible = gfx::IntRect(0,
                                         0,
@@ -600,7 +600,7 @@ AppleVTDecoder::CreateOutputConfiguration()
   if (mUseSoftwareImages) {
     // Output format type:
     SInt32 PixelFormatTypeValue =
-      kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange;
+      kCVPixelFormatType_420YpCbCr8Planar;
     AutoCFRelease<CFNumberRef> PixelFormatTypeNumber =
       CFNumberCreate(kCFAllocatorDefault,
                      kCFNumberSInt32Type,
